@@ -164,6 +164,8 @@ class Client(object):
         self.name = six.text_type(o.get('name') or defaults.NAME)
         self.auto_log_stacks = bool(
             o.get('auto_log_stacks') or defaults.AUTO_LOG_STACKS)
+        self.capture_locals = bool(
+            o.get('capture_locals', defaults.CAPTURE_LOCALS))
         self.string_max_length = int(
             o.get('string_max_length') or defaults.MAX_LENGTH_STRING)
         self.list_max_length = int(
@@ -296,11 +298,13 @@ class Client(object):
             else:
                 frames = stack
 
+            stack_info = get_stack_info(
+                frames,
+                transformer=self.transform,
+                capture_locals=self.capture_locals,
+            )
             data.update({
-                'sentry.interfaces.Stacktrace': {
-                    'frames': get_stack_info(frames,
-                        transformer=self.transform)
-                },
+                'sentry.interfaces.Stacktrace': stack_info,
             })
 
         if 'sentry.interfaces.Stacktrace' in data:
@@ -353,7 +357,7 @@ class Client(object):
             data.update(processor.process(data))
 
         if 'message' not in data:
-            data['message'] = handler.to_string(data)
+            data['message'] = kwargs.get('message', handler.to_string(data))
 
         # tags should only be key=>u'value'
         for key, value in six.iteritems(data['tags']):
