@@ -1,6 +1,6 @@
 """
 raven.utils.compat
-~~~~~~~~~~~~~~~~~~~~~~~~~~
+~~~~~~~~~~~~~~~~~~
 
 :copyright: (c) 2010-2012 by the Sentry Team, see AUTHORS for more details.
 :license: BSD, see LICENSE for more details.
@@ -15,10 +15,12 @@ except ImportError:
 
 
 try:
-    from urllib.request import Request, urlopen
+    import urllib.request as urllib2
 except ImportError:
-    from urllib2 import Request, urlopen  # NOQA
+    import urllib2
 
+Request = urllib2.Request
+urlopen = urllib2.urlopen
 
 try:
     from urllib import quote as urllib_quote
