@@ -2,37 +2,45 @@
 
 from __future__ import absolute_import
 from __future__ import with_statement
+from __future__ import unicode_literals
 
 import datetime
 import django
 import logging
 import mock
+import pytest
 import re
+import sys  # NOQA
 from exam import fixture
-from celery.tests.utils import with_eager_tasks
-from StringIO import StringIO
 
 from django.conf import settings
+from django.core.exceptions import SuspiciousOperation
 from django.core.urlresolvers import reverse
 from django.core.signals import got_request_exception
 from django.core.handlers.wsgi import WSGIRequest
+from django.http import QueryDict
 from django.template import TemplateSyntaxError
 from django.test import TestCase
 
 from raven.base import Client
-from raven.contrib.django import DjangoClient
+from raven.contrib.django.client import DjangoClient
 from raven.contrib.django.celery import CeleryClient
 from raven.contrib.django.handlers import SentryHandler
 from raven.contrib.django.models import client, get_client, sentry_exception_handler
 from raven.contrib.django.middleware.wsgi import Sentry
+from raven.contrib.django.templatetags.raven import sentry_public_dsn
 from raven.contrib.django.views import is_valid_origin
 from raven.utils.serializer import transform
+from raven.utils import six
+from raven.utils.six import StringIO
 
 from django.test.client import Client as TestClient, ClientHandler as TestClientHandler
 from .models import TestModel
 
 settings.SENTRY_CLIENT = 'tests.contrib.django.tests.TempStoreClient'
 
+DJANGO_15 = django.VERSION >= (1, 5, 0)
+
 
 def make_request():
     return WSGIRequest(environ={
@@ -69,6 +77,11 @@ class TempStoreClient(DjangoClient):
         return True
 
 
+class DisabledTempStoreClient(TempStoreClient):
+    def is_enabled(self, **kwargs):
+        return False
+
+
 class Settings(object):
     """
     Allows you to define settings that are required for this function to work.
@@ -84,12 +97,12 @@ class Settings(object):
         self._orig = {}
 
     def __enter__(self):
-        for k, v in self.overrides.iteritems():
+        for k, v in six.iteritems(self.overrides):
             self._orig[k] = getattr(settings, k, self.NotDefined)
             setattr(settings, k, v)
 
     def __exit__(self, exc_type, exc_value, traceback):
-        for k, v in self._orig.iteritems():
+        for k, v in six.iteritems(self._orig):
             if v is self.NotDefined:
                 delattr(settings, k)
             else:
@@ -137,9 +150,9 @@ class DjangoClientTest(TestCase):
         self.assertTrue('sentry.interfaces.Exception' in event)
         exc = event['sentry.interfaces.Exception']
         self.assertEquals(exc['type'], 'ValueError')
-        self.assertEquals(exc['value'], u"invalid literal for int() with base 10: 'hello'")
+        self.assertEquals(exc['value'], "invalid literal for int() with base 10: 'hello'")
         self.assertEquals(event['level'], logging.ERROR)
-        self.assertEquals(event['message'], u"ValueError: invalid literal for int() with base 10: 'hello'")
+        self.assertEquals(event['message'], "ValueError: invalid literal for int() with base 10: 'hello'")
         self.assertEquals(event['culprit'], 'tests.contrib.django.tests in test_signal_integration')
 
     def test_view_exception(self):
@@ -163,29 +176,48 @@ class DjangoClientTest(TestCase):
 
         self.assertRaises(Exception, self.client.get, reverse('sentry-raise-exc'))
 
-        self.assertEquals(len(self.raven.events), 1)
+        assert len(self.raven.events) == 1
         event = self.raven.events.pop(0)
-        self.assertTrue('sentry.interfaces.User' in event)
-        user_info = event['sentry.interfaces.User']
-        self.assertTrue('is_authenticated' in user_info)
-        self.assertFalse(user_info['is_authenticated'])
-        self.assertFalse('username' in user_info)
-        self.assertFalse('email' in user_info)
+        assert 'sentry.interfaces.User' not in event
 
-        self.assertTrue(self.client.login(username='admin', password='admin'))
+        assert self.client.login(username='admin', password='admin')
 
         self.assertRaises(Exception, self.client.get, reverse('sentry-raise-exc'))
 
         self.assertEquals(len(self.raven.events), 1)
         event = self.raven.events.pop(0)
-        self.assertTrue('sentry.interfaces.User' in event)
+        assert 'sentry.interfaces.User' in event
         user_info = event['sentry.interfaces.User']
-        self.assertTrue('is_authenticated' in user_info)
-        self.assertTrue(user_info['is_authenticated'])
-        self.assertTrue('username' in user_info)
-        self.assertEquals(user_info['username'], 'admin')
-        self.assertTrue('email' in user_info)
-        self.assertEquals(user_info['email'], 'admin@example.com')
+        assert user_info == {
+            'is_authenticated': True,
+            'username': user.username,
+            'id': user.id,
+            'email': user.email,
+        }
+
+    @pytest.mark.skipif(str('not DJANGO_15'))
+    def test_get_user_info_abstract_user(self):
+        from django.db import models
+        from django.contrib.auth.models import AbstractBaseUser
+
+        class MyUser(AbstractBaseUser):
+            USERNAME_FIELD = 'username'
+
+            username = models.CharField(max_length=32)
+            email = models.EmailField()
+
+        user = MyUser(
+            username='admin',
+            email='admin@example.com',
+            id=1,
+        )
+        user_info = self.raven.get_user_info(user)
+        assert user_info == {
+            'is_authenticated': True,
+            'username': user.username,
+            'id': user.id,
+            'email': user.email,
+        }
 
     def test_request_middleware_exception(self):
         with Settings(MIDDLEWARE_CLASSES=['tests.contrib.django.middleware.BrokenRequestMiddleware']):
@@ -264,7 +296,7 @@ class DjangoClientTest(TestCase):
 
     def test_exclude_modules_view(self):
         exclude_paths = self.raven.exclude_paths
-        self.raven.exclude_paths = ['tests.views.decorated_raise_exc']
+        self.raven.exclude_paths = ['tests.views']
         self.assertRaises(Exception, self.client.get, reverse('sentry-raise-exc-decor'))
 
         self.assertEquals(len(self.raven.events), 1)
@@ -275,14 +307,14 @@ class DjangoClientTest(TestCase):
 
     def test_include_modules(self):
         include_paths = self.raven.include_paths
-        self.raven.include_paths = ['django.shortcuts.get_object_or_404']
+        self.raven.include_paths = ['django.shortcuts']
 
         self.assertRaises(Exception, self.client.get, reverse('sentry-django-exc'))
 
         self.assertEquals(len(self.raven.events), 1)
         event = self.raven.events.pop(0)
 
-        self.assertEquals(event['culprit'], 'django.shortcuts in get_object_or_404')
+        assert event['culprit'].startswith('django.shortcuts in ')
         self.raven.include_paths = include_paths
 
     def test_template_name_as_view(self):
@@ -303,6 +335,8 @@ class DjangoClientTest(TestCase):
     #     self.assertEquals(event['culprit'], 'tests.contrib.django.views in logging_request_exc')
     #     self.assertEquals(event['data']['META']['REMOTE_ADDR'], '127.0.0.1')
 
+    # TODO: Python bug #10805
+    @pytest.mark.skipif(str('six.PY3'))
     def test_record_none_exc_info(self):
         # sys.exc_info can return (None, None, None) if no exception is being
         # handled anywhere on the stack. See:
@@ -321,7 +355,6 @@ class DjangoClientTest(TestCase):
 
         self.assertEquals(len(self.raven.events), 1)
         event = self.raven.events.pop(0)
-
         self.assertEquals(event['message'], 'test')
 
     def test_404_middleware(self):
@@ -337,11 +370,34 @@ class DjangoClientTest(TestCase):
 
             self.assertTrue('sentry.interfaces.Http' in event)
             http = event['sentry.interfaces.Http']
-            self.assertEquals(http['url'], u'http://testserver/non-existant-page')
+            self.assertEquals(http['url'], 'http://testserver/non-existant-page')
             self.assertEquals(http['method'], 'GET')
             self.assertEquals(http['query_string'], '')
             self.assertEquals(http['data'], None)
 
+    def test_404_middleware_when_disabled(self):
+        extra_settings = {
+            'MIDDLEWARE_CLASSES': ['raven.contrib.django.middleware.Sentry404CatchMiddleware'],
+            'SENTRY_CLIENT': 'tests.contrib.django.tests.DisabledTempStoreClient',
+        }
+        with Settings(**extra_settings):
+            resp = self.client.get('/non-existant-page')
+            assert resp.status_code == 404
+            assert self.raven.events == []
+
+    def test_invalid_client(self):
+        extra_settings = {
+            'SENTRY_CLIENT': 'raven.contrib.django.DjangoClient',  # default
+        }
+        # Should return fallback client (TempStoreClient)
+        client = get_client('nonexistant.and.invalid')
+
+        # client should be valid, and the same as with the next call.
+        assert client is get_client()
+
+        with Settings(**extra_settings):
+            assert isinstance(get_client(), DjangoClient)
+
     def test_response_error_id_middleware(self):
         # TODO: test with 500s
         with Settings(MIDDLEWARE_CLASSES=['raven.contrib.django.middleware.SentryResponseErrorIdMiddleware',
@@ -352,7 +408,7 @@ class DjangoClientTest(TestCase):
             self.assertTrue('X-Sentry-ID' in headers)
             self.assertEquals(len(self.raven.events), 1)
             event = self.raven.events.pop(0)
-            self.assertEquals('$'.join([event['event_id'], event['checksum']]), headers['X-Sentry-ID'])
+            assert event['event_id'] == headers['X-Sentry-ID']
 
     def test_get_client(self):
         self.assertEquals(get_client(), get_client())
@@ -363,10 +419,7 @@ class DjangoClientTest(TestCase):
             self.raven)
         self.assertEquals(get_client(), self.raven)
 
-    # This test only applies to Django 1.3+
     def test_raw_post_data_partial_read(self):
-        if django.VERSION[:2] < (1, 3):
-            return
         v = '{"foo": "bar"}'
         request = make_request()
         request.environ.update({
@@ -386,10 +439,22 @@ class DjangoClientTest(TestCase):
         self.assertEquals(http['method'], 'POST')
         self.assertEquals(http['data'], '<unavailable>')
 
-    # This test only applies to Django 1.3+
+    def test_read_post_data(self):
+        request = make_request()
+        request.POST = QueryDict("foo=bar&ham=spam")
+        request.read(1)
+
+        self.raven.captureMessage(message='foo', request=request)
+
+        self.assertEquals(len(self.raven.events), 1)
+        event = self.raven.events.pop(0)
+
+        self.assertTrue('sentry.interfaces.Http' in event)
+        http = event['sentry.interfaces.Http']
+        self.assertEquals(http['method'], 'POST')
+        self.assertEquals(http['data'], {'foo': 'bar', 'ham': 'spam'})
+
     def test_request_capture(self):
-        if django.VERSION[:2] < (1, 3):
-            return
         request = make_request()
         request.read(1)
 
@@ -418,7 +483,7 @@ class DjangoClientTest(TestCase):
         self.assertEquals(len(self.raven.events), 1)
         event = self.raven.events.pop(0)
 
-        frames = event['sentry.interfaces.Stacktrace']['frames']
+        frames = event['sentry.interfaces.Exception']['stacktrace']['frames']
         for frame in frames:
             if frame['module'].startswith('django.'):
                 assert frame.get('in_app') is False
@@ -431,7 +496,51 @@ class DjangoClientTest(TestCase):
 
         tags = event['tags']
         assert 'site' in event['tags']
-        assert tags['site'] == u'example.com'
+        assert tags['site'] == 'example.com'
+
+    def test_adds_site_to_tags_fallback(self):
+        with Settings(SITE_ID=12345):  # nonexistant site, should fallback to SITE_ID
+            self.assertRaises(TemplateSyntaxError, self.client.get, reverse('sentry-template-exc'))
+
+            self.assertEquals(len(self.raven.events), 1)
+            event = self.raven.events.pop(0)
+
+            tags = event['tags']
+            assert 'site' in event['tags']
+            assert tags['site'] == 12345
+
+    def test_settings_site_overrides_contrib(self):
+        self.raven.site = 'FOO'
+        self.assertRaises(TemplateSyntaxError, self.client.get, reverse('sentry-template-exc'))
+
+        self.assertEquals(len(self.raven.events), 1)
+        event = self.raven.events.pop(0)
+
+        tags = event['tags']
+        assert 'site' in event['tags']
+        assert tags['site'] == 'FOO'
+
+    @mock.patch.object(WSGIRequest, 'build_absolute_uri')
+    def test_suspicious_operation_in_build_absolute_uri(self, build_absolute_uri):
+        build_absolute_uri.side_effect = SuspiciousOperation()
+        request = make_request()
+        request.META['HTTP_HOST'] = 'example.com'
+        result = self.raven.get_data_from_request(request)
+        build_absolute_uri.assert_called_once_with()
+        assert 'sentry.interfaces.Http' in result
+        assert result['sentry.interfaces.Http']['url'] == 'http://example.com/'
+
+
+class DjangoTemplateTagTest(TestCase):
+    @mock.patch('raven.contrib.django.DjangoClient.get_public_dsn')
+    def test_sentry_public_dsn_no_args(self, get_public_dsn):
+        sentry_public_dsn()
+        get_public_dsn.assert_called_once_with(None)
+
+    @mock.patch('raven.contrib.django.DjangoClient.get_public_dsn')
+    def test_sentry_public_dsn_with_https(self, get_public_dsn):
+        sentry_public_dsn('https')
+        get_public_dsn.assert_called_once_with('https')
 
 
 class DjangoLoggingTest(TestCase):
@@ -488,17 +597,6 @@ class CeleryIsolatedClientTest(TestCase):
 
         self.assertEquals(send_raw.delay.call_count, 1)
 
-    @with_eager_tasks
-    @mock.patch('raven.contrib.django.DjangoClient.send_encoded')
-    def test_with_eager(self, send_encoded):
-        """
-        Integration test to ensure it propagates all the way down
-        and calls the parent client's send_encoded method.
-        """
-        self.client.captureMessage(message='test')
-
-        self.assertEquals(send_encoded.call_count, 1)
-
 
 class CeleryIntegratedClientTest(TestCase):
     def setUp(self):
@@ -522,17 +620,6 @@ class CeleryIntegratedClientTest(TestCase):
 
             self.assertEquals(send_raw.delay.call_count, 1)
 
-    @with_eager_tasks
-    @mock.patch('raven.contrib.django.DjangoClient.send_encoded')
-    def test_with_eager(self, send_encoded):
-        """
-        Integration test to ensure it propagates all the way down
-        and calls the parent client's send_encoded method.
-        """
-        self.client.captureMessage(message='test')
-
-        self.assertEquals(send_encoded.call_count, 1)
-
 
 class IsValidOriginTestCase(TestCase):
     def test_setting_empty(self):
@@ -610,7 +697,8 @@ class ReportViewTest(TestCase):
     @mock.patch('raven.contrib.django.views.is_valid_origin', mock.Mock(return_value=True))
     def test_sends_x_sentry_auth_header(self):
         resp = self.client.post(self.path, HTTP_ORIGIN='http://example.com',
-            HTTP_X_SENTRY_AUTH='Sentry foo/bar', data='{}', content_type='application/octet-stream')
+            HTTP_X_SENTRY_AUTH='Sentry foo/bar', data='{}',
+            content_type='application/octet-stream')
         self.assertEquals(resp.status_code, 200)
         event = client.events.pop(0)
         self.assertEquals(event, {'auth_header': 'Sentry foo/bar'})
@@ -620,38 +708,42 @@ class PromiseSerializerTestCase(TestCase):
     def test_basic(self):
         from django.utils.functional import lazy
 
-        obj = lazy(lambda: 'bar', str)()
+        obj = lazy(lambda: 'bar', six.text_type)()
         res = transform(obj)
-        self.assertEquals(res, 'bar')
+        expected = "'bar'" if six.PY3 else "u'bar'"
+        self.assertEquals(res, expected)
 
     def test_handles_gettext_lazy(self):
         from django.utils.functional import lazy
 
         def fake_gettext(to_translate):
-            return u'Igpay Atinlay'
+            return 'Igpay Atinlay'
 
-        fake_gettext_lazy = lazy(fake_gettext, str)
+        fake_gettext_lazy = lazy(fake_gettext, six.text_type)
 
         result = transform(fake_gettext_lazy("something"))
-        self.assertTrue(isinstance(result, basestring))
-        self.assertEquals(result, u'Igpay Atinlay')
+        self.assertTrue(isinstance(result, six.string_types))
+        expected = "'Igpay Atinlay'" if six.PY3 else "u'Igpay Atinlay'"
+        self.assertEquals(result, expected)
 
 
-class QuerySetSerializerTestCase(TestCase):
-    def test_model_instance(self):
+class ModelInstanceSerializerTestCase(TestCase):
+    def test_basic(self):
         instance = TestModel()
 
         result = transform(instance)
-        self.assertTrue(isinstance(result, basestring))
-        self.assertEquals(result, u'<TestModel: TestModel object>')
+        self.assertTrue(isinstance(result, six.string_types))
+        self.assertEquals(result, '<TestModel: TestModel object>')
 
+
+class QuerySetSerializerTestCase(TestCase):
     def test_basic(self):
         from django.db.models.query import QuerySet
         obj = QuerySet(model=TestModel)
 
         result = transform(obj)
-        self.assertTrue(isinstance(result, basestring))
-        self.assertEquals(result, u'<QuerySet: model=TestModel>')
+        self.assertTrue(isinstance(result, six.string_types))
+        self.assertEquals(result, '<QuerySet: model=TestModel>')
 
 
 class SentryExceptionHandlerTest(TestCase):
@@ -681,3 +773,29 @@ class SentryExceptionHandlerTest(TestCase):
         sentry_exception_handler(request=self.request)
 
         assert not captureException.called
+
+    @mock.patch.object(TempStoreClient, 'captureException')
+    @mock.patch('sys.exc_info')
+    @mock.patch('raven.contrib.django.models.get_option')
+    def test_ignore_exceptions_with_expression_match(self, get_option, exc_info, captureException):
+        exc_info.return_value = self.exc_info
+        get_option.return_value = ['builtins.*']
+        if not six.PY3:
+            get_option.return_value = ['exceptions.*']
+
+        sentry_exception_handler(request=self.request)
+
+        assert not captureException.called
+
+    @mock.patch.object(TempStoreClient, 'captureException')
+    @mock.patch('sys.exc_info')
+    @mock.patch('raven.contrib.django.models.get_option')
+    def test_ignore_exceptions_with_module_match(self, get_option, exc_info, captureException):
+        exc_info.return_value = self.exc_info
+        get_option.return_value = ['builtins.ValueError']
+        if not six.PY3:
+            get_option.return_value = ['exceptions.ValueError']
+
+        sentry_exception_handler(request=self.request)
+
+        assert not captureException.called
