@@ -1,6 +1,9 @@
+from __future__ import unicode_literals
+
 import logging
 import sys
-from unittest2 import TestCase
+from raven.utils.testutils import TestCase
+from raven.utils import six
 from raven.base import Client
 from raven.handlers.logging import SentryHandler
 from raven.utils.stacks import iter_stack_frames
@@ -23,10 +26,10 @@ class LoggingIntegrationTest(TestCase):
         self.client = TempStoreClient(include_paths=['tests', 'raven'])
         self.handler = SentryHandler(self.client)
 
-    def make_record(self, msg, args=(), level=logging.INFO, extra=None, exc_info=None):
-        record = logging.LogRecord('root', level, __file__, 27, msg, args, exc_info, 'make_record')
+    def make_record(self, msg, args=(), level=logging.INFO, extra=None, exc_info=None, name='root', pathname=__file__):
+        record = logging.LogRecord(name, level, pathname, 27, msg, args, exc_info, 'make_record')
         if extra:
-            for key, value in extra.iteritems():
+            for key, value in six.iteritems(extra):
                 record.__dict__[key] = value
         return record
 
@@ -34,17 +37,29 @@ class LoggingIntegrationTest(TestCase):
         record = self.make_record('This is a test error')
         self.handler.emit(record)
 
-        self.assertEquals(len(self.client.events), 1)
+        self.assertEqual(len(self.client.events), 1)
         event = self.client.events.pop(0)
-        self.assertEquals(event['logger'], 'root')
-        self.assertEquals(event['level'], logging.INFO)
-        self.assertEquals(event['message'], 'This is a test error')
-        self.assertFalse('sentry.interfaces.Stacktrace' in event)
+        self.assertEqual(event['logger'], 'root')
+        self.assertEqual(event['level'], logging.INFO)
+        self.assertEqual(event['message'], 'This is a test error')
         self.assertFalse('sentry.interfaces.Exception' in event)
         self.assertTrue('sentry.interfaces.Message' in event)
         msg = event['sentry.interfaces.Message']
-        self.assertEquals(msg['message'], 'This is a test error')
-        self.assertEquals(msg['params'], ())
+        self.assertEqual(msg['message'], 'This is a test error')
+        self.assertEqual(msg['params'], ())
+
+    def test_can_record(self):
+        tests = [
+            ("raven", False),
+            ("raven.foo", False),
+            ("sentry.errors", False),
+            ("sentry.errors.foo", False),
+            ("raven_utils", True),
+        ]
+
+        for test in tests:
+            record = self.make_record("Test", name=test[0])
+            self.assertEqual(self.handler.can_record(record), test[1])
 
     def test_logger_extra_data(self):
         record = self.make_record('This is a test error', extra={'data': {
@@ -52,9 +67,13 @@ class LoggingIntegrationTest(TestCase):
         }})
         self.handler.emit(record)
 
-        self.assertEquals(len(self.client.events), 1)
+        self.assertEqual(len(self.client.events), 1)
         event = self.client.events.pop(0)
-        self.assertEquals(event['extra']['url'], 'http://example.com')
+        if six.PY3:
+            expected = "'http://example.com'"
+        else:
+            expected = "u'http://example.com'"
+        self.assertEqual(event['extra']['url'], expected)
 
     def test_logger_exc_info(self):
         try:
@@ -66,100 +85,120 @@ class LoggingIntegrationTest(TestCase):
 
         self.handler.emit(record)
 
-        self.assertEquals(len(self.client.events), 1)
+        self.assertEqual(len(self.client.events), 1)
         event = self.client.events.pop(0)
 
-        self.assertEquals(event['message'], 'This is a test info with an exception')
-        self.assertTrue('sentry.interfaces.Stacktrace' in event)
+        self.assertEqual(event['message'], 'This is a test info with an exception')
         self.assertTrue('sentry.interfaces.Exception' in event)
         exc = event['sentry.interfaces.Exception']
-        self.assertEquals(exc['type'], 'ValueError')
-        self.assertEquals(exc['value'], 'This is a test ValueError')
+        self.assertEqual(exc['type'], 'ValueError')
+        self.assertEqual(exc['value'], 'This is a test ValueError')
         self.assertTrue('sentry.interfaces.Message' in event)
         msg = event['sentry.interfaces.Message']
-        self.assertEquals(msg['message'], 'This is a test info with an exception')
-        self.assertEquals(msg['params'], ())
+        self.assertEqual(msg['message'], 'This is a test info with an exception')
+        self.assertEqual(msg['params'], ())
 
     def test_message_params(self):
         record = self.make_record('This is a test of %s', args=('args',))
         self.handler.emit(record)
 
-        self.assertEquals(len(self.client.events), 1)
+        self.assertEqual(len(self.client.events), 1)
         event = self.client.events.pop(0)
-        self.assertEquals(event['message'], 'This is a test of args')
+        self.assertEqual(event['message'], 'This is a test of args')
         msg = event['sentry.interfaces.Message']
-        self.assertEquals(msg['message'], 'This is a test of %s')
-        self.assertEquals(msg['params'], ('args',))
+        self.assertEqual(msg['message'], 'This is a test of %s')
+        expected = ("'args'",) if six.PY3 else ("u'args'",)
+        self.assertEqual(msg['params'], expected)
 
     def test_record_stack(self):
         record = self.make_record('This is a test of stacks', extra={'stack': True})
         self.handler.emit(record)
 
-        self.assertEquals(len(self.client.events), 1)
+        self.assertEqual(len(self.client.events), 1)
         event = self.client.events.pop(0)
         self.assertTrue('sentry.interfaces.Stacktrace' in event)
         frames = event['sentry.interfaces.Stacktrace']['frames']
         self.assertNotEquals(len(frames), 1)
         frame = frames[0]
-        self.assertEquals(frame['module'], 'raven.handlers.logging')
+        self.assertEqual(frame['module'], 'raven.handlers.logging')
         self.assertFalse('sentry.interfaces.Exception' in event)
         self.assertTrue('sentry.interfaces.Message' in event)
-        self.assertEquals(event['culprit'], 'root in make_record')
-        self.assertEquals(event['message'], 'This is a test of stacks')
+        self.assertEqual(event['culprit'], 'root in make_record')
+        self.assertEqual(event['message'], 'This is a test of stacks')
 
     def test_no_record_stack(self):
         record = self.make_record('This is a test with no stacks', extra={'stack': False})
         self.handler.emit(record)
 
-        self.assertEquals(len(self.client.events), 1)
+        self.assertEqual(len(self.client.events), 1)
         event = self.client.events.pop(0)
-        self.assertEquals(event['message'], 'This is a test with no stacks')
+        self.assertEqual(event['message'], 'This is a test with no stacks')
         self.assertFalse('sentry.interfaces.Stacktrace' in event)
 
     def test_explicit_stack(self):
         record = self.make_record('This is a test of stacks', extra={'stack': iter_stack_frames()})
         self.handler.emit(record)
 
-        self.assertEquals(len(self.client.events), 1)
+        self.assertEqual(len(self.client.events), 1)
         event = self.client.events.pop(0)
         assert 'sentry.interfaces.Stacktrace' in event
         assert 'culprit' in event
         assert event['culprit'] == 'root in make_record'
         self.assertTrue('message' in event, event)
-        self.assertEquals(event['message'], 'This is a test of stacks')
+        self.assertEqual(event['message'], 'This is a test of stacks')
         self.assertFalse('sentry.interfaces.Exception' in event)
         self.assertTrue('sentry.interfaces.Message' in event)
         msg = event['sentry.interfaces.Message']
-        self.assertEquals(msg['message'], 'This is a test of stacks')
-        self.assertEquals(msg['params'], ())
+        self.assertEqual(msg['message'], 'This is a test of stacks')
+        self.assertEqual(msg['params'], ())
 
     def test_extra_culprit(self):
         record = self.make_record('This is a test of stacks', extra={'culprit': 'foo in bar'})
         self.handler.emit(record)
 
-        self.assertEquals(len(self.client.events), 1)
+        self.assertEqual(len(self.client.events), 1)
         event = self.client.events.pop(0)
-        self.assertEquals(event['culprit'], 'foo in bar')
+        self.assertEqual(event['culprit'], 'foo in bar')
 
     def test_extra_data_as_string(self):
         record = self.make_record('Message', extra={'data': 'foo'})
         self.handler.emit(record)
 
-        self.assertEquals(len(self.client.events), 1)
+        self.assertEqual(len(self.client.events), 1)
         event = self.client.events.pop(0)
-        self.assertEquals(event['extra']['data'], 'foo')
+        expected = "'foo'" if six.PY3 else "u'foo'"
+        self.assertEqual(event['extra']['data'], expected)
+
+    def test_tags(self):
+        record = self.make_record('Message', extra={'tags': {'foo': 'bar'}})
+        self.handler.emit(record)
+
+        self.assertEqual(len(self.client.events), 1)
+        event = self.client.events.pop(0)
+        assert event['tags'] == {'foo': 'bar'}
+
+    def test_tags_on_error(self):
+        try:
+            raise ValueError('This is a test ValueError')
+        except ValueError:
+            record = self.make_record('Message', extra={'tags': {'foo': 'bar'}}, exc_info=sys.exc_info())
+        self.handler.emit(record)
+
+        self.assertEqual(len(self.client.events), 1)
+        event = self.client.events.pop(0)
+        assert event['tags'] == {'foo': 'bar'}
 
 
 class LoggingHandlerTest(TestCase):
     def test_client_arg(self):
         client = TempStoreClient(include_paths=['tests'])
         handler = SentryHandler(client)
-        self.assertEquals(handler.client, client)
+        self.assertEqual(handler.client, client)
 
     def test_client_kwarg(self):
         client = TempStoreClient(include_paths=['tests'])
         handler = SentryHandler(client=client)
-        self.assertEquals(handler.client, client)
+        self.assertEqual(handler.client, client)
 
     def test_args_as_servers_and_key(self):
         handler = SentryHandler(['http://sentry.local/api/store/'], 'KEY')
@@ -183,4 +222,4 @@ class LoggingHandlerTest(TestCase):
 
     def test_logging_level_not_set(self):
         handler = SentryHandler('http://public:secret@example.com/1')
-        self.assertEquals(handler.level, logging.NOTSET)
+        self.assertEqual(handler.level, logging.NOTSET)
